#!/usr/bin/perl
#
# migrate_smsd.conf.pl
#
# Used to migrate old smsd.conf files created with op5 APS 3.x to
# op5 APS 6. This is only supposed to be executed during a migration
# inside op5-backup. 
# 
# Settings that comes from the original smsd.conf delivered with
# smstools 3
#
# devices=GSM1
# logfile=/var/log/smsd/smsd.log
# infofile=/var/run/smsd/smsd.working
# pidfile=/var/run/smsd/smsd.pid
# loglevel=7
# user=smstools
# # 3.1.5 introduced smart logging
# # once your configuration is OK, set log level lower (5 is good in most cases)
# smart_logging=yes
#

use Config::Simple;
use Data::Dumper;
use List::MoreUtils qw(uniq);
use File::Temp (tempdir);
use File::Copy;

my $login = (getpwuid $>);
die "must run as root" if $login ne 'root';

my $debug = 0;

# Create temporary working directory
# It will be deleted when the script exits.
my $tmp_conf_dir = File::Temp->newdir( TEMPLATE => 'op5smsdXXXXXXX',
                                    DIR => '/tmp/');
$tmp_conf_dir->unlink_on_destroy( 1 );

print "TMP conf dir: $tmp_conf_dir\n\n" if $debug;

move ('/etc/smsd.conf', $tmp_conf_dir . "/smsd.conf.orig") or die ($!);
my $old_smsd_file = $tmp_conf_dir . "/smsd.conf.orig";


# Getting all the old settings.
my $conf = new Config::Simple($old_smsd_file);

# Get the blocks from the old smsd.conf
my %config = $conf->vars();
my @block;

my $n = 0;
foreach $keys (sort {$a <=> $b} %config) {
	if ($config{$keys} ne '') {
		my $stuff = $keys;
		$stuff =~ s/(.*)\.(.*)//g;
		$block[$n] = $1;
		$n++;
		print "Found block: $1\n" if $debug;
	}
}


# Get the settings for each block and save it
# to a temporay file. One file for each block.
my %smsdconf;
foreach $bk (uniq(@block)) {
	$smsdconf{$bk} = new Config::Simple(syntax=>'ini');

	foreach $keys (sort {$a <=> $b} %config) {
		if (($config{$keys} ne '') && ($keys =~ /^$bk/)) {
			$smsdconf{$bk}->param($keys, $config{$keys});
			print "Adding data to \"$bk\": " . $keys . ", " . $config{$keys} . "\n" if $debug;
		}
	}
	# The default block will be handled separatly since it's
	# here we need to change some settings to follow
	# smstoosl on CentOS as much as possible.
	if ($bk =~ /^default/) {
		$smsdconf{$bk}->param('default.logfile', '/var/log/smsd/smsd.log');
		$smsdconf{$bk}->param('default.infofile', '/var/run/smsd/smsd.working');
		$smsdconf{$bk}->param('default.pidfile', '/var/run/smsd/smsd.pid');
		$smsdconf{$bk}->param('default.loglevel', '5');
		$smsdconf{$bk}->param('default.user', 'root');
		$smsdconf{$bk}->param('default.smart_logging', 'yes');
		$smsdconf{$bk}->param('default.stats', '/var/log/smsd_stats');
		$smsdconf{$bk}->param('default.stats_interval', '3600');
		$smsdconf{$bk}->delete('default.mypath');
	}
	# Write the temporary block config file.
	$smsdconf{$bk}->write($tmp_conf_dir->dirname . "/" . $bk);
	print "Writing data to: " . $tmp_conf_dir->dirname . "/" . $bk . "\n" if $debug;
}

# Get a list of all blocks by their file names
opendir(DIR, $tmp_conf_dir->dirname);
my @cfgdir = grep { (!/^\./) } readdir(DIR);
closedir(DIR);

if ($debug) {
	foreach (@cfgdir) {
		print "Found the following block files:\n";
		print "  " . $_ . "\n";
	}
}

# Write the default block at the top of the smsd.conf
# file. Remove stupid lines starting with ; that are
# added by Config::Simple->write.
# Since smsd.conf should not include the [default]
# block header it will be excluded as well.
open (SMSDCONF, ">> " . $tmp_conf_dir . "/smsd.conf");
open (FILE, "< " . $tmp_conf_dir . "/default");
while (<FILE>) {
	if (! /^;|^\[/) {
		print SMSDCONF $_;
	}
}
close(FILE);

# Write the rest of the block files into smsd.conf
# after the default block. Also remove the lines that
# starts with ;.
foreach (sort @cfgdir) {
	if (! /default|smsd\.conf/) {
		my $infile = $tmp_conf_dir . "/" . $_;
		open (INFILE, "< " . $infile);
		while (<INFILE>) {
			if (! /^;/) {
				print SMSDCONF $_;
				print "Writing from " . $infile . " to " . $tmp_conf_dir . "/smsd.conf: " . $_  if $debug;
			}
		}
		close(INFILE);
	}
}
print SMSDCONF "\n";
close(SMSDCONF);

# Put the new smsd.conf i place.
print "Moving " . $tmp_conf_dir . "/smsd.conf to /etc/smsd.conf\n" if $debug;
move ($tmp_conf_dir . "/smsd.conf", "/etc/smsd.conf") or die ($!);
